@@ -33,7 +33,14 @@ typedef struct TAG_MK_VECTOR
 	unsigned int flags;
 	unsigned int size;
 	unsigned int used;
-	void**		 items;
+	union
+	{
+		INT_PTR*	items;
+		INT8 *		items8;
+		INT16*		items16;
+		INT32*		items32;
+		INT64*		items64;
+	};
 } MK_VECTOR;
 
 // memory pool.
@@ -46,30 +53,32 @@ typedef struct TAG_MK_MEMORY_POOL
 
 // --- code object
 
-// function code object
-typedef struct TAG_MK_FUNCTION
-{
-	unsigned int flags;
-	MK_CHAR *name;
-	union
-	{
-		MK_VECTOR* args;				// script
-		unsigned int sizeArgs;			// native
-	};
-	union
-	{
-		struct TAG_MK_NODE_BLOCK *impl;	// script
-		void *entryPoint;				// native
-	};
-}MK_FUNCTION;
-
 // variable definition object
 struct TAG_MK_NODE_EXPR;
 typedef struct TAG_MK_VARIABLE
 {
 	unsigned int flags;
 	MK_CHAR *name;
-	struct TAG_MK_NODE_EXPR *defaultValue;
+	union
+	{
+		// for variable
+		struct TAG_MK_NODE_EXPR *defaultValue;
+		
+		// for method
+		struct
+		{
+			union
+			{
+				MK_VECTOR* args;				// script
+				unsigned int sizeArgs;			// native
+			};
+			union
+			{
+				struct TAG_MK_NODE_BLOCK *impl;	// script
+				void *entryPoint;				// native
+			};
+		};
+	};
 } MK_VARIABLE;
 
 // class code object
@@ -79,30 +88,34 @@ typedef struct TAG_MK_CLASS
 	MK_CHAR *nameThis;
 	MK_CHAR *nameSuper;
 	MK_HASHTABLE* variables;
-	MK_HASHTABLE* functions;
-	MK_VECTOR* operatorFunctions;
+	INT_PTR *operatorMethod;
 } MK_CLASS;
 
 // expr code object
 typedef struct TAG_MK_NODE_EXPR
 {
 	unsigned int flags;
+
+	void *stream;
+	unsigned int line;
+
 	union
 	{
-		MK_CHAR *symbolName;				// function_call, symbol, new, delete
+		MK_CHAR *symbolName;				// method_call, symbol, new, delete
 		MK_CHAR *typeName;					// MK_VM_VARIABLE_INSTANCE
 		MK_VECTOR *multipleSymbols;			// multiple symbol top
+		MK_VECTOR *arrayDefinition;			// array definition.
 		unsigned int length;				// string
 		unsigned int constantValue;			// constant_value(high)
 		struct TAG_MK_NODE_EXPR *left;		// operation, return
+		unsigned int *node;					// node_with_param(MK_VARIABLE)
 	}u1;
 	union
 	{
 		struct TAG_MK_NODE_EXPR *right;		// operation
 		unsigned int constantValue;			// constant_value(low)
 		void *value;						// string
-		MK_VECTOR *args;					// function_call, new
-		MK_HASHTABLE *variables;			// MK_VM_VARIABLE_INSTANCE( class instance )
+		MK_VECTOR *args;					// method_call, new
 	}u2;
 } MK_NODE_EXPR;
 
@@ -130,19 +143,12 @@ typedef struct TAG_MK_NODE_WHILE
 	struct TAG_MK_NODE_BLOCK *block;	// internal block
 }MK_NODE_WHILE;
 
-// node with parameter object
-typedef struct TAG_MK_NODE_WITH_PARAM
-{
-	unsigned int flags;
-	MK_VECTOR *param;
-	unsigned int *node;		// node : MK_NODE_EXPR, MK_NODE_BLOCK, MK_NODE_IF, MK_NODE_WHILE
-} MK_NODE_WITH_PARAM;
-
 // catch block struct
 typedef struct TAG_MK_CATCH_BLOCK
 {
-	MK_NODE_WITH_PARAM *paramCatch;
-	MK_NODE_BLOCK *blockCatch;
+	unsigned int flags;
+	MK_NODE_EXPR  *paramCatch;	// MK_TYPE_NODE_EXPR_NODE_WITH_PARAM
+	MK_NODE_EXPR *blockCatch;
 } MK_CATCH_BLOCK;
 
 // try ... ( catch ... )+ finally ...
@@ -165,17 +171,62 @@ typedef struct TAG_MK_VM_FRAME_ITEM_CLASS
 	MK_HASHTABLE *variables;
 } MK_VM_FRAME_ITEM_CLASS;
 
+struct TAG_MK_VM_FRAME;
 typedef struct TAG_MK_VM_FRAME_ITEM
 {
 	unsigned int flags;
 	union
 	{
-		int int32TypeValue;
-		MK_CHAR *stringTypeValue;
-		MK_FLOAT floatTypeValue;
-		MK_VM_FRAME_ITEM_CLASS classTypeValue;
+		// integer
+		union	
+		{
+			struct
+			{
+				INT32 int32TypeValue;
+				INT32 int32TypeHighValue;
+			};
+			INT64 int64TypeValue;
+		};
+		
+		// string
+		struct	
+		{
+			MK_CHAR *stringTypeValue;
+			INT32 sizeString;
+		};
+
+		// float
+		MK_FLOAT floatTypeValue;	
+
+		// class
+		struct
+		{
+			MK_CHAR *typeName;
+			struct TAG_MK_VM_FRAME_ITEM *child;
+			MK_HASHTABLE *variables;
+		} classTypeValue;
+		
+		// array
+		MK_VECTOR *arrayTypeValue;
+
+		// node
+		struct
+		{
+			struct TAG_MK_VM_FRAME *definedFrame;
+			struct TAG_MK_VM_FRAME_ITEM *pOwner;
+			unsigned int *node;
+		} code;
+
+		// native object
+		struct
+		{
+			void *nativePtr;
+			void (*native_free)(void);
+			INT32 sizeNativePtr;
+		} nativeObjectTypeValue;
+
+		// reference.
 		struct TAG_MK_VM_FRAME_ITEM *reference;
-		unsigned int *code;
 	};
 } MK_VM_FRAME_ITEM;
 
@@ -184,7 +235,8 @@ typedef struct TAG_MK_VM_FRAME
 	unsigned int flags;
 	MK_VM_FRAME_ITEM *pThis;
 	MK_HASHTABLE *localVariables;
-	MK_FUNCTION *pFunction;
+	MK_VARIABLE *pMethod;
+	struct TAG_MK_VM_FRAME *pOwnerFrame;
 	struct TAG_MK_VM_FRAME *previous;
 } MK_VM_FRAME;
 
@@ -223,7 +275,7 @@ typedef struct TAG_MK_MANAGED_VECTOR_TABLE
 	unsigned int freeSpace;
 	unsigned int nextIndex;
 	struct TAG_MK_MANAGED_VECTOR_TABLE *previous;
-	MK_VECTOR *vectors[MK_SIZEOF_MANAGED_TABLE];
+	MK_VECTOR vectors[MK_SIZEOF_MANAGED_TABLE];
 } MK_MANAGED_VECTOR_TABLE;
 
 
@@ -233,8 +285,6 @@ typedef struct TAG_MK_MANAGED_VECTOR_TABLE
 typedef struct TAG_MK_STREAM
 {
 	void *stream;
-	MK_CHAR *name;
-	unsigned int line;
 	unsigned char current;
 	MK_CHAR back;
 	void*(*open)(void*);
@@ -247,10 +297,14 @@ typedef struct TAG_MK_LEXER
 {
 	MK_STREAM stream;	// stream access interface
 	
+	MK_CHAR *name;
+
+	unsigned int nextLine;
 	MK_CHAR *nextText;
 	unsigned int nextLength;
 	unsigned int nextValue;
 
+	unsigned int line;
 	MK_CHAR *text;
 	unsigned int length;
 	unsigned int value;
@@ -279,6 +333,14 @@ typedef struct TAG_MK_VM_CACHE
 	MK_CLASS			*pConstantClass[0x0f];	// internal class
 } MK_VM_CACHE;
 
+// ---- vm local stack
+typedef struct TAG_MK_VM_STACK
+{
+	unsigned int	**start;
+	unsigned int	**current;
+	unsigned int	**last;
+} MK_VM_STACK;
+
 // ---- vm core
 typedef struct TAG_MK_VM_STRUCT
 {
@@ -293,14 +355,17 @@ typedef struct TAG_MK_VM_STRUCT
 
 	MK_OBJECTCODE *code;
 
-	MK_HASHTABLE *global;
-	MK_VM_FRAME *pTopFrame;
-	MK_VM_FRAME *pCurrentFrame;
-	MK_VECTOR *localStack;
+	// vm execute environment.
+	MK_HASHTABLE *global;			// global variables
+	MK_VM_FRAME *pTopFrame;			// top frame
+	MK_VM_FRAME *pCurrentFrame;		// current frame
+
+	MK_VM_STACK localStack;		// stack
 
 	MK_MANAGED_VM_FRAME_ITEM_TABLE *pFrameItemTable;
 	MK_MANAGED_VM_FRAME_TABLE *pFrameTable;
 	MK_MANAGED_HASH_TABLE *pHashTable;
+	MK_MANAGED_VECTOR_TABLE *pVectorTable;
 
 	MK_VM_FRAME_ITEM *exceptionObject;	// current exception object
 
@@ -319,21 +384,20 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_OBJECT_TYPE_VM_FRAME				0x30000000		// MK_VM_FRAME
 #define MK_OBJECT_TYPE_VM_FRAME_ITEM		0x40000000		// MK_VM_FRAME_ITEM(managed by gc)
 #define MK_TYPE_CLASS						0x50000000		// MK_CLASS
-#define MK_TYPE_FUNCTION					0x60000000		// MK_FUNCTION
-#define MK_TYPE_VARIABLE					0x70000000		// MK_VARIABLE
-#define MK_TYPE_NODE_BLOCK					0x80000000		// MK_NODE_BLOCK
-#define MK_TYPE_NODE_IF						0x90000000		// MK_NODE_IF
-#define MK_TYPE_NODE_WHILE					0xa0000000		// MK_NODE_WHILE
-#define MK_TYPE_TRY_BLOCK					0xb0000000		// MK_TRY_BLOCK
+#define MK_TYPE_VARIABLE					0x60000000		// MK_VARIABLE
+#define MK_TYPE_NODE_BLOCK					0x70000000		// MK_NODE_BLOCK
+#define MK_TYPE_NODE_IF						0x80000000		// MK_NODE_IF
+#define MK_TYPE_NODE_WHILE					0x90000000		// MK_NODE_WHILE
+#define MK_TYPE_TRY_BLOCK					0xa0000000		// MK_TRY_BLOCK
+#define MK_TYPE_CATCH_BLOCK					0xb0000000		// MK_CATCH_BLOCK
 #define MK_TYPE_NODE_EXPR					0xc0000000		// MK_NODE_EXPR
 #define MK_TYPE_HASHTABLE					0xd0000000		// MK_HASHTABLE
 #define MK_TYPE_VECTOR						0xe0000000		// MK_VECTOR
-#define MK_TYPE_NODE_WITH_PARAM				0xf0000000		// MK_NODE_WITH_PARAM
 
 // object attribute
 #define MK_TYPE_ATTRIBUTE_MASK					0x0fe00000
 #define MK_TYPE_ATTRIBUTE(value)				( (value) & MK_TYPE_ATTRIBUTE_MASK )
-#define MK_TYPE_SET_ATTRIBUTE(target, value)	( ( target & ~MK_TYPE_ATTRIBUTE_MASK ) | MK_TYPE_ATTRIBUTE(value) )
+#define MK_TYPE_SET_ATTRIBUTE(target, value)	( ( (target) & ~MK_TYPE_ATTRIBUTE_MASK ) | MK_TYPE_ATTRIBUTE(value) )
 #define MK_TYPE_ATTRIBUTE_INDEX(value)			( MK_TYPE_ATTRIBUTE(value) >> 21 )
 
 // attribute of MK_OBJECT_TYPE_VM_STRUCT
@@ -342,18 +406,20 @@ typedef struct TAG_MK_VM_STRUCT
 
 // attribute of MK_OBJECT_TYPE_VM_FRAME
 #define MK_VM_FRAME_TYPE_ROOT					0x00000000
-#define MK_VM_FRAME_TYPE_PARAM_WIDH_NODE		0x01000000
+#define MK_VM_FRAME_TYPE_NODE_WITH_PARAM		0x01000000
 
 // attribute of MK_OBJECT_TYPE_VM_FRAME_ITEM
 #define MK_VM_FRAME_ITEM_TYPE_NIL				0x00000000
 #define MK_VM_FRAME_ITEM_TYPE_CLASS				0x01000000
 #define MK_VM_FRAME_ITEM_TYPE_INT_VALUE			0x02000000
 #define MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE		0x03000000
+#define MK_VM_FRAME_ITEM_TYPE_ARRAY_VALUE		0x04000000
 #define MK_VM_FRAME_ITEM_TYPE_STRING_VALUE		0x05000000
 #define MK_VM_FRAME_ITEM_TYPE_CHARACTER_VALUE	0x06000000
 #define MK_VM_FRAME_ITEM_TYPE_BOOL_VALUE		0x07000000
-#define MK_VM_FRAME_ITEM_TYPE_CODE_VALUE		0x08000000
+#define MK_VM_FRAME_ITEM_TYPE_NATIVE_PTR		0x08000000
 #define MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE	0x09000000
+#define MK_VM_FRAME_ITEM_TYPE_NODE_VALUE		0x0a000000
 #define MK_VM_FRAME_ITEM_TYPE_THIS				0x0f200000
 #define MK_VM_FRAME_ITEM_TYPE_SUPER				0x0f400000
 #define MK_VM_FRAME_ITEM_TYPE_OWNER				0x0f600000
@@ -366,6 +432,10 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_VM_FRAME_ITEM_TYPE_WRITE_PRIVATE		0x000c0000
 
 #define MK_VM_FRAME_ITEM_TYPE_FINAL				0x00008000
+#define MK_VM_FRAME_ITEM_TYPE_STATIC_INSTANCE	0x00004000
+
+// special frame element.
+#define MK_VM_FRAME_SPECIAL_FRAME_MASK			0x0000000f
 
 // MK_VM_FRAME_ITEM_PTR value
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE		0x00000001
@@ -379,32 +449,37 @@ typedef struct TAG_MK_VM_STRUCT
 // attribute of MK_TYPE_CLASS
 
 // attribute of MK_TYPE_FUNCTION
-#define MK_TYPE_ATTRIBUTE_FUNCTION_NORMAL		0x00000000
-#define MK_TYPE_ATTRIBUTE_FUNCTION_CONSTRUCTER	0x01000000
-#define MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR		0x03000000
-
-#define MK_TYPE_ATTRIBUTE_FUNCTION_STATIC		0x04000000
-#define MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE		0x08000000
-
-#define MK_TYPE_ATTRIBUTE_FUNCTION_PRIVATE		0x00400000
-#define MK_TYPE_ATTRIBUTE_FUNCTION_PUBLIC		0x00800000
-#define MK_TYPE_ATTRIBUTE_FUNCTION_PROTECTED	0x00C00000
-
-#define MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR_ID		0x000fc000
 
 // attribute of MK_TYPE_VARIABLE
-#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PUBLIC		0x01000000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PROTECTED	0x02000000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PRIVATE		0x03000000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PUBLIC		0x04000000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PROTECTED	0x08000000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PRIVATE	0x0c000000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_VALUE			0x00000000	// variable type
+#define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD			0x08000000	// metod type
 
-#define MK_TYPE_ATTRIBUTE_VARIALBE_FINAL			0x00800000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_MEMBER			0x00400000
-#define MK_TYPE_ATTRIBUTE_VARIABLE_STATIC_MEMBER	0x00200000
+// style only method
+#define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_SCRIPT	0x00000000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE	0x04000000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR	0x02000000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_VARARGS	0x01000000
 
-#define MK_TYPE_ATTRIBUTE_VARIABLE_CALL_BY_VALUE	0x00100000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_STATIC			0x00800000
+
+
+#define MK_TYPE_ATTRIBUTE_VARIABLE_FINAL			0x00400000	// variable type.
+#define MK_TYPE_ATTRIBUTE_VARIABLE_VIRTUAL			0x00400000	// method type.
+
+// read access style
+#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PUBLIC		0x00100000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PROTECTED	0x00200000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_READ_PRIVATE		0x00300000
+
+// write access style( variable only ).
+#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PUBLIC		0x0004000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PROTECTED	0x00080000
+#define MK_TYPE_ATTRIBUTE_VARIABLE_WRITE_PRIVATE	0x000c0000
+
+#define MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR_ID		0x0004f000
+
+// method arguments param(testing)
+#define MK_TYPE_ATTRIBUTE_VARIABLE_CALL_BY_VALUE	0x00000800
 
 // attribute of MK_TYPE_BLOCK
 #define MK_TYPE_ATTRIBUTE_BLOCK_FUNCTION_ROOT		0x00000000
@@ -413,7 +488,7 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_ATTRIBUTE_BLOCK_TRY_BLOCK			0x03000000
 #define MK_TYPE_ATTRIBUTE_BLOCK_CATCH_BLOCK			0x04000000
 #define MK_TYPE_ATTRIBUTE_BLOCK_FINALLY_BLOCK		0x05000000
-#define MK_TYPE_ATTRIBUTE_BLOCK_NODE_WITH_PARAM		0x06000000
+#define MK_TYPE_ATTRIBUTE_BLOCK_NONAME_METHOD		0x06000000
 
 // attribute of MK_TYPE_IF
 
@@ -434,12 +509,17 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_NODE_EXPR_DBLATSYMBOL		0x04000000
 #define MK_TYPE_NODE_EXPR_MULTIPLESYMBOL	0x05000000
 #define MK_TYPE_NODE_EXPR_FUNCTION_CALL		0x06000000
+#define MK_TYPE_NODE_EXPR_FUNCTION_CALL_INSTANCE	0x0C200000
+#define MK_TYPE_NODE_EXPR_FUNCTION_CALL_STATIC		0x0C400000
 #define MK_TYPE_NODE_EXPR_SUPER_CALL		0x06600000	
 #define MK_TYPE_NODE_EXPR_RETURN			0x07000000
 #define MK_TYPE_NODE_EXPR_BREAK				0x08000000
 #define MK_TYPE_NODE_EXPR_CONTINUE			0x09000000
 #define MK_TYPE_NODE_EXPR_NEW				0x0a000000
-#define MK_TYPE_NODE_EXPR_RAISE				0x0d000000
+#define MK_TYPE_NODE_EXPR_RAISE				0x0b000000
+#define MK_TYPE_NODE_EXPR_NODE_WITH_PARAM	0x0c000000
+
+#define MK_TYPE_NODE_EXPR_ARRAY_DEFINITION	0x0d000000
 
 #define MK_TYPE_NODE_EXPR_CONSTANT		0x00e00000
 #define MK_TYPE_NODE_EXPR_INT8			0x03e00000
@@ -454,9 +534,6 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_NODE_EXPR_OPERATIONTYPE_MASK		0x000fc000
 #define MK_TYPE_NODE_EXPR_OPERATIONPRIORITY_MASK	0x00003c00
 
-#define MK_TYPE_NODE_EXPR_OPERATION_LEFT_CONST		0x00000200
-#define MK_TYPE_NODE_EXPR_OPERATION_RIGHT_CONST		0x00000100
-
 #define MK_MAKE_LEX_RESERVED_MARK( priority, index, isLeftConst, isRightConst )	\
 	( MK_LEX_TYPE_RESERVED_MARK + ( ( priority ) << 10 ) + ( ( index ) << 14 ) + ( isLeftConst << 9 ) + ( isRightConst << 8 ) )
 #define MK_LEX_RESERVED_MARK_INDEX( id )	\
@@ -496,39 +573,58 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_MARK_SMALL		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 7, 17, 1, 1 )
 #define MK_LEX_TYPE_RESERVED_MARK_EQUAL		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 8, 18, 0, 1 )
 #define MK_LEX_TYPE_RESERVED_MARK_DOT		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 19, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_DBLATMARK	MK_MAKE_LEX_RESERVED_MARK( 0, 20, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_ATMARK	MK_MAKE_LEX_RESERVED_MARK( 0, 21, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_PARENTHESIS		MK_MAKE_LEX_RESERVED_MARK( 0, 22, 0, 0 )	// (
-#define MK_LEX_TYPE_RESERVED_MARK_END_PARENTHESIS	MK_MAKE_LEX_RESERVED_MARK( 0, 23, 0, 0 )	// )
-#define MK_LEX_TYPE_RESERVED_MARK_BRACKET	MK_MAKE_LEX_RESERVED_MARK( 0, 24, 0, 0 )			// [
-#define MK_LEX_TYPE_RESERVED_MARK_END_BRACKET	MK_MAKE_LEX_RESERVED_MARK( 0, 25, 0, 0 )		// ]
-#define MK_LEX_TYPE_RESERVED_MARK_BRACE	MK_MAKE_LEX_RESERVED_MARK( 0, 26, 0, 0 )				// {
-#define MK_LEX_TYPE_RESERVED_MARK_END_BRACE	MK_MAKE_LEX_RESERVED_MARK( 0, 27, 0, 0 )			// }
+#define MK_LEX_TYPE_RESERVED_MARK_PARENTHESIS		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 20, 0, 0 )	// (
+#define MK_LEX_TYPE_RESERVED_MARK_END_PARENTHESIS	MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 21, 0, 0 )	// )
+#define MK_LEX_TYPE_RESERVED_MARK_BRACKET			MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 22, 0, 0 )	// [
+#define MK_LEX_TYPE_RESERVED_MARK_END_BRACKET		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 23, 0, 0 )	// ]
+#define MK_LEX_TYPE_RESERVED_MARK_BRACE				MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 24, 0, 0 )	// {
+#define MK_LEX_TYPE_RESERVED_MARK_END_BRACE			MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 25, 0, 0 )	// }
+
+#define MK_LEX_TYPE_RESERVED_MARK_DBLATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 26, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_ATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 27, 0, 0 )
+
 #define MK_LEX_TYPE_RESERVED_MARK_EXCLAMATION	MK_MAKE_LEX_RESERVED_MARK( 0, 28, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_QUESTION	MK_MAKE_LEX_RESERVED_MARK( 0, 29, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CORON		MK_MAKE_LEX_RESERVED_MARK( 0, 30, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_QUESTION		MK_MAKE_LEX_RESERVED_MARK( 0, 29, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CORON			MK_MAKE_LEX_RESERVED_MARK( 0, 30, 0, 0 )
 #define MK_LEX_TYPE_RESERVED_MARK_DOUBLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 31, 0, 0 )
 #define MK_LEX_TYPE_RESERVED_MARK_SINGLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 32, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CONMA		MK_MAKE_LEX_RESERVED_MARK( 0, 33, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CHILDA	MK_MAKE_LEX_RESERVED_MARK( 0, 34, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_SHARP		MK_MAKE_LEX_RESERVED_MARK( 0, 35, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CONMA			MK_MAKE_LEX_RESERVED_MARK( 0, 33, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CHILDA		MK_MAKE_LEX_RESERVED_MARK( 0, 34, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_SHARP			MK_MAKE_LEX_RESERVED_MARK( 0, 35, 0, 0 )
 
-#define MK_SIZEOF_ENABELD_OPELATORS	20
+#define MK_SIZEOF_ENABELD_OPELATORS	26
 
 // hashtable
-#define MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK	0x00ffc000	// 0 ~ 1023
-#define MK_TYPE_HASHTABLE_SIZE_EXTEND(flag)	((flag & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK)>>14)
-#define MK_TYPE_ATTRIBUTE_HASHKEY_STYLE(flag)	( (flag) & 0x03000000 )
+#define MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK	0x000ffc00	// 0 ~ 1023
+#define MK_TYPE_HASHTABLE_SIZE_EXTEND(flag)	((flag & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK)>>10)
+
+#define MK_TYPE_ATTRIBUTE_HASHKEY_STYLE(flag)	( (flag) & 0x01000000 )
 #define MK_TYPE_ATTRIBUTE_HASHKEY_MASK			0x03000000
 #define MK_TYPE_ATTRIBUTE_HASHKEY_STRING		0x00000000
-#define MK_TYPE_ATTRIBUTE_HASHKEY_INT32			0x01000000
-#define MK_TYPE_ATTRIBUTE_HASHKEY_INT64			0x02000000
-#define MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR		0x03000000
+#define MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR		0x01000000
+#define MK_TYPE_ATTRIBUTE_HASHKEY_MANAGEDPTR	0x02000000
 
+#define MK_TYPE_ATTRIBUTE_HASHVALUE_STYLE(flag)	( (flag) & 0x02000000 )
+#define MK_TYPE_ATTRIBUTE_HASHVALUE_MASK		0x0C000000
+#define MK_TYPE_ATTRIBUTE_HASHVALUE_MANAGEDPTR	0x00000000
+#define MK_TYPE_ATTRIBUTE_HASHVALUE_UNMANAGED	0x04000000
 
 //vector
-#define MK_TYPE_VECTOR_SIZE_EXTEND_MASK		0x00ffc000	// 0 ~ 1023
-#define MK_TYPE_VECTOR_SIZE_EXTEND(flag)	((flag & MK_TYPE_VECTOR_SIZE_EXTEND_MASK)>>14)
+#define MK_TYPE_VECTOR_SIZE_EXTEND_MASK		0x000ffc00	// 0 ~ 1023
+#define MK_TYPE_VECTOR_SIZE_EXTEND(flag)	((flag & MK_TYPE_VECTOR_SIZE_EXTEND_MASK)>>10)
+
+#define MK_TYPE_ATTRIBUTE_VECTOR_STYLE(flag)	( (flag) & 0x0f000000 )
+#define MK_TYPE_ATTRIBUTE_VECTOR_STYLE_MASK		0x0f000000
+#define MK_TYPE_VECTOR_ARRAY_MANAGED_PTR		0x00000000	// need destroy_node
+#define MK_TYPE_VECTOR_ARRAY_UNMANAGED_PTR		0x01000000	// need not destroy_node
+#define MK_TYPE_VECTOR_ARRAY_INT8				0x02000000
+#define MK_TYPE_VECTOR_ARRAY_INT16				0x03000000
+#define MK_TYPE_VECTOR_ARRAY_INT32				0x04000000
+#define MK_TYPE_VECTOR_ARRAY_INT64				0x05000000
+
+#define MK_TYPE_VECTOR_SIZE_FIX					0x00800000
+#define MK_TYPE_VECTOR_POINT_MEMORY_DIRECT		0x00400000
+
 
 // gc bit
 #define MK_TYPE_GC_MASk				0x00000003
@@ -602,6 +698,7 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_SYMBOL_READ		MK_LEX_TYPE_RESERVED_SYMBOL + 26
 #define MK_LEX_TYPE_RESERVED_SYMBOL_WRITE		MK_LEX_TYPE_RESERVED_SYMBOL + 27
 #define MK_LEX_TYPE_RESERVED_SYMBOL_INITIALIZE	MK_LEX_TYPE_RESERVED_SYMBOL + 28
+#define MK_LEX_TYPE_RESERVED_SYMBOL_BLOCK		MK_LEX_TYPE_RESERVED_SYMBOL + 29
 
 #define MK_LEX_STRING_MARK					'\"'
 #define MK_LEX_CHARACTER_MARK				'\''
@@ -666,7 +763,7 @@ int mk_is_ptr_in_memory_pool ( const void *ptr, MK_MEMORY_POOL *top );
 
 // raise critical error
 int mk_get_internal_error_message( MK_CHAR *name, int line, MK_CHAR *buffer, size_t size, int errorNo, va_list list );
-MK_VM_FRAME_ITEM *mk_create_internal_error_object( MK_VM_STRUCT *vm, MK_CHAR *nameStream, int line, MK_CLASS *classException, int errorNo, va_list list );
+MK_VM_FRAME_ITEM *mk_create_internal_error_object( MK_VM_STRUCT *vm, MK_CHAR *nameStream, int line, MK_VM_FRAME_ITEM *classException, int errorNo, va_list list );
 int mk_raise_internal_error( MK_VM_STRUCT *vm, MK_CHAR *name, int line, int errorNo, ... );
 
 // lexer
@@ -680,48 +777,38 @@ int do_compile( MK_VM_STRUCT *vm, void*target );
 // dynamic code generate
 int mk_register_class( MK_VM_STRUCT *vm, MK_CLASS *target );
 int mk_register_variable( MK_VM_STRUCT *vm, MK_VARIABLE *targetVariable, MK_CLASS *targetClass );
-int mk_register_function( MK_VM_STRUCT *vm, MK_FUNCTION *targetFunction, MK_CLASS *targetClass );
-int mk_register_function_arg( MK_VM_STRUCT *vm, MK_VECTOR *targetArgs, MK_VARIABLE *arg );
-MK_FUNCTION *mk_create_native_function( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int sizeArgs, unsigned int type, void *fp );
+int mk_register_variable_arg( MK_VM_STRUCT *vm, MK_VECTOR *targetArgs, MK_VARIABLE *arg );
 MK_VARIABLE *mk_create_variable( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int type, MK_NODE_EXPR *defaultValue );
+MK_VARIABLE *mk_create_method( MK_VM_STRUCT *vm, MK_CHAR *name, unsigned int type, INT_PTR arg, INT_PTR impl );
+
+// dump object(for debug)
+void dump_node( void *target, unsigned int rank );
 
 // vm
+int mk_vm_execute_expr_default_value( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pOwner, MK_VARIABLE *valueVariable, MK_VM_FRAME_ITEM *defaultValue );
 int mk_vm_initialize( MK_VM_STRUCT *vm );
+void mk_register_internal_classes( MK_VM_STRUCT *vm );
 int mk_vm_run( MK_VM_STRUCT *vm );
-unsigned int mk_native_function_call( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction, unsigned int sizeParam );
+int mk_execute_block( MK_VM_STRUCT *vm, unsigned int *target );
+unsigned int mk_native_method_call( MK_VM_STRUCT *vm, MK_VARIABLE *pMethod, unsigned int sizeParam );
 int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target );
 MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target );
 MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value );
 MK_VM_FRAME_ITEM *mk_vm_create_float_frame_item( MK_VM_STRUCT *vm, MK_FLOAT value );
 MK_VM_FRAME_ITEM *mk_vm_create_bool_frame_item( int isTrue );
 int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target );
-#ifdef _DEBUG
-void mk_vm_push_stack( MK_VECTOR *stack, MK_VM_FRAME_ITEM *target );
-MK_VM_FRAME_ITEM* mk_vm_pop_stack( MK_VECTOR *stack );
-#else
-#define mk_vm_push_stack	mk_push_vector
-#define mk_vm_pop_stack		mk_pop_vector
-#endif
-void mk_destroy_vm_frame_item( MK_MEMORY_POOL *top, MK_VM_FRAME_ITEM *target );
-#ifdef _DEBUG
-int mk_execute_expr( MK_VM_STRUCT *vm, MK_NODE_EXPR *pTarget, int hasParent );
-#else
-#define mk_execute_expr( vm, pTarget, hasParent )													\
-	( ( abs( (INT_PTR)vm->stackTop - (INT_PTR)&vm ) < MK_SIZEOF_MAX_STACK_SIZE ) ?		\
-		vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX((pTarget)->flags)](vm, pTarget, hasParent ) :		\
-		mk_raise_internal_error( vm, "", 0, MK_ERROR_TYPE_VM_ERROR | 11, NULL ) )	
-#endif
-MK_CLASS *mk_vm_get_class_by_name( MK_VM_STRUCT *vm, MK_CHAR **pname );
-MK_CLASS *mk_vm_get_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget );
-int mk_vm_call_function( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction );
 
-#ifdef _DEBUG
-#define MK_VM_FRAME_ITEM_VARIDATE( target )	\
-	( MK_GC_IS_ALLOCATED_OBJECT( target->flags ) != 0 )
-#else
-#define MK_VM_FRAME_ITEM_VARIDATE( target )	\
-	1
-#endif
+void mk_destroy_vm_frame_item( MK_MEMORY_POOL *top, MK_VM_FRAME_ITEM *target );
+
+#include "vm_stack.h"
+
+#define mk_execute_expr( vm, pTarget, hasParent )														\
+( ( abs( (INT_PTR)((vm)->stackTop) - (INT_PTR)&(vm) ) < MK_SIZEOF_MAX_STACK_SIZE ) ?					\
+		vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX((pTarget)->flags)]((vm), (pTarget), (hasParent) ) :	\
+		mk_raise_internal_error( vm, "", 0, MK_ERROR_TYPE_VM_ERROR | 9, NULL ) )
+MK_CLASS *mk_vm_get_class_by_name( MK_VM_STRUCT *vm, const MK_CHAR *pname );
+MK_CLASS *mk_vm_get_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget );
+int mk_vm_call_method( MK_VM_STRUCT *vm, MK_VARIABLE *pMethod, MK_VM_FRAME *pOwnerFrame );
 
 #define mk_get_entity_vm_frame_item( item )			\
 {													\
@@ -730,70 +817,77 @@ int mk_vm_call_function( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction );
 		item = item->reference;						\
 }
 
+// special class method
+int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result );
+int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result );
+
 // object create/destroy
 void *mk_create_object( unsigned int nsType );
 void mk_destroy_node( MK_MEMORY_POOL *memoryPool, void *target );
 MK_VM_FRAME *mk_create_vm_frame_object( MK_MANAGED_VM_FRAME_TABLE **frameTable );
 MK_VM_FRAME_ITEM *mk_create_vm_frame_item_object( MK_MANAGED_VM_FRAME_ITEM_TABLE **itemTable );
 
-// dump object(for debug)
-void dump_node( void *target, unsigned int rank );
-
 // vector
-MK_VECTOR *mk_create_vector( unsigned int defaultSize, unsigned int extend );
-unsigned int mk_insert_at_vector( MK_VECTOR **ptarget, unsigned int index, void *elem );
 void mk_destroy_vector_node( MK_MEMORY_POOL *top, MK_VECTOR *target );
-void *mk_set_at_vector( MK_VECTOR *ptarget, unsigned int index, void *newItem );
+MK_VECTOR *mk_create_vector( unsigned int flag, unsigned int defaultSize, INT_PTR addressOfMemory );
+MK_VECTOR *mk_allocate_vm_managed_vector( MK_MANAGED_VECTOR_TABLE **managedVectorTable, 
+										 unsigned int flag, unsigned int defaultSize, INT_PTR addressOfMemory );
+unsigned int mk_insert_at_vector( MK_VECTOR *target, unsigned int index, INT_PTR elem );
+INT_PTR mk_set_at_vector( MK_VECTOR *ptarget, unsigned int index, INT_PTR newItem );
+INT_PTR mk_get_at_vector( MK_VECTOR *ptarget, unsigned int index );
 
-#ifdef _DEBUG
-unsigned int mk_size_vector( MK_VECTOR *ptarget );
-void* mk_get_at_vector( MK_VECTOR *ptarget, unsigned int index );
-void mk_trim_size_vector( MK_VECTOR *ptarget, unsigned int newSize );
-const void **mk_get_item_ptr_vector( MK_VECTOR *ptarget );
-void mk_push_vector( MK_VECTOR *ptarget, void *elem );
-void* mk_pop_vector( MK_VECTOR *ptarget );
-#else
-#define mk_get_item_ptr_vector( ptarget )							\
-	( ( ptarget != NULL ) ? (const void**)ptarget->items : NULL )
+
+#define mk_get_item_ptr_vector( ptarget )				\
+	( (const INT_PTR*)ptarget->items )
 
 #define mk_trim_size_vector( ptarget, newSize )			\
-	if( ptarget != NULL && ptarget->used > newSize )	\
-		ptarget->used = newSize;
+{														\
+	if ( ptarget->used > newSize )						\
+		ptarget->used = newSize;						\
+}
 
-#define mk_push_vector( ptarget, elem )						\
-	if(ptarget!=NULL && ptarget->used < ptarget->size )		\
-		ptarget->items[ptarget->used++] = elem;				\
-	else													\
-		mk_insert_at_vector( &ptarget, 0xffffffff, elem );
+#define mk_push_vector( ptarget, elem )					\
+	mk_insert_at_vector( ptarget, UINT_MAX, elem )
 
-#define mk_size_vector( ptarget )			\
-	( ( ptarget != NULL ) ? ptarget->used : 0 )
+#define mk_size_vector( ptarget )						\
+	( ptarget->used )
 
-#define mk_get_at_vector( ptarget, index )	\
-	( ( ptarget != NULL && index < ptarget->used ) ? ptarget->items[index] : NULL )
-
-#define mk_pop_vector( ptarget )	\
-	( ( ptarget != NULL && ptarget->used > 0 ) ? ptarget->items[--ptarget->used] : NULL )
-#endif
+#define mk_pop_vector( ptarget )						\
+	mk_get_at_vector( ptarget, --ptarget->used )
 
 const MK_CHAR *symbol_int_to_string( unsigned int value );
 const MK_CHAR *operation_int_to_string( unsigned int value );
-MK_FUNCTION* mk_vm_find_function( MK_VM_STRUCT *vm, MK_CLASS *pOwner, const MK_CHAR *name, unsigned int sizeArgs );
-MK_FUNCTION* mk_vm_find_operator_function( MK_VM_STRUCT *vm, MK_CLASS *pOwner, unsigned int index );
+MK_VARIABLE* mk_vm_find_method( MK_VM_STRUCT *vm, 
+								 MK_VM_FRAME_ITEM *pOwner, const MK_CHAR *name, 
+								 unsigned int sizeArgs );
+MK_VM_FRAME_ITEM* mk_vm_find_variable( MK_VM_STRUCT *vm, 
+								 MK_VM_FRAME_ITEM *pOwner, const MK_CHAR *name );
 
 // gc
 int mk_gc_run( MK_VM_STRUCT *vm, int force );
 
-#define CLASS_INTERNAL_KERNEL "Kernel"
 #define CLASS_INTERNAL_OBJECT	"Object"
 #define CLASS_INTERNAL_NULL		"Null"
+
+#define CLASS_INTERNAL_KERNEL "Kernel"
+
 #define CLASS_INTERNAL_INTEGER	"Integer"
 #define	CLASS_INTERNAL_FLOAT	"Float"
+
 #define CLASS_INTERNAL_IO		"IO"
 #define CLASS_INTERNAL_CONSOLE	"Console"
+
+#define CLASS_INTERNAL_CONTAINER	"Container"
+#define CLASS_INTERNAL_ARRAY	"Array"
+
+#define CLASS_INTERNAL_NODE		"Node"
+
 #define CLASS_INTERNAL_EXCEPTION "Exception"
 #define CLASS_INTERNAL_COMPILE_ERROR_EXCEPTION	"CompileErrorException"
 #define CLASS_INTERNAL_VM_ERROR_EXCEPTION	"VmErrorException"
+#define CLASS_INTERNAL_ARRAY_OUTOF_RANGE	"ArrayOutOfRangeException"
+
+
 
 #define FUNCTIONNAME_ENTRYPOINT "mk_main"
 
