@@ -28,8 +28,21 @@
 #define INT16		short
 #define INT32		int
 #define INT64		long long
-#define INT_PTR		int			// 32bit
+#define INT_PTR		int			// 32bit / 64bit
 
+#ifndef _MSC_VER
+#define UINT_MAX	( (INT_PTR)0 - 1 )
+#endif
+
+#ifdef _MSC_VER
+#define INLINE	__inline
+#else
+#define INLINE	static inline
+#endif
+
+// vm stack size
+#define MK_SIZEOF_VM_STACK				64 * 1024
+#define MK_SIZEOF_EXTEND_STACK			0	// no extend
 
 // table size
 #define MK_SIZEOF_VECTOR_DEFAULT		16
