@@ -11,31 +11,32 @@ int mk_console_print( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_IT
 	if( (INT_PTR)display & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE ||
 		MK_TYPE_ATTRIBUTE( display->flags ) != MK_VM_FRAME_ITEM_TYPE_STRING_VALUE )
 	{
-		MK_FUNCTION *pFunction = NULL;
-		MK_CLASS *pClass = NULL;
-		pClass = 
-			mk_vm_get_class( vm, display );
-		if( pClass == NULL )
-			return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
+		MK_VARIABLE *pMethod = NULL;
+		int retCode = 0;
 
-		pFunction = 
-			mk_vm_find_function( vm, 
-			pClass, 
-			mk_get_symbol_name_ptr( vm, "to_s" ), 
-			0 );
-		if( pFunction == NULL )
-			return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
-		mk_vm_push_stack( vm->localStack, display );
-		if( mk_vm_call_function( vm, pFunction ) == MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
+		// call display.to_s()
+		pMethod = 
+			mk_vm_find_method( vm, 
+				display, 
+				mk_get_symbol_name_ptr( vm, "to_s" ), 
+				0 );
+		mk_vm_push_stack( &vm->localStack, (unsigned int*)display );
+		retCode = 
+			mk_vm_call_method( vm, pMethod, NULL );
+		if( retCode == MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
 		{
 			MK_VM_FRAME_ITEM *converted = 
-				mk_get_at_vector( vm->localStack, mk_size_vector( vm->localStack ) - 1 );
+				(MK_VM_FRAME_ITEM*)mk_vm_pop_stack( &vm->localStack );
 			if( converted != NULL && 
 				MK_TYPE_ATTRIBUTE( converted->flags ) == MK_VM_FRAME_ITEM_TYPE_STRING_VALUE )
 				mk_console_print( vm, target, converted, result );
 			else
 				fputs( "Unknown object", stdout );
 		}
+		else
+		{
+			return MK_VM_EXECUTE_EXPR_THROW;
+		}
 	}
 	else
 	{
@@ -80,44 +81,58 @@ MK_CLASS *mk_create_console_class( MK_VM_STRUCT *vm )
 	result->nameThis = mk_get_symbol_name_ptr( vm, className );
 	result->nameSuper = mk_get_symbol_name_ptr( vm, classSuper );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"print",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_STATIC, 
-			mk_console_print ),
+			(INT_PTR)mk_console_print ),
 		result );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"println",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_STATIC, 
-			mk_console_print_ln ),
+			(INT_PTR)mk_console_print_ln ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
 			"<<", 
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_LSHIFT, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_LSHIFT, 
-			mk_console_lshift ),
+			(INT_PTR)mk_console_lshift ),
 		result);
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"getc",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_STATIC, 
-			mk_console_getc ),
+			(INT_PTR)mk_console_getc ),
 		result );
 	return result;
-}
\ No newline at end of file
+}
