@@ -32,7 +32,7 @@ static
 int mk_int_to_f( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result )
 {
 	*result = 
-		mk_vm_create_float_frame_item( vm, mk_vm_frame_item_to_float(target) );
+		mk_vm_create_int32_frame_item( vm, mk_vm_frame_item_to_int32(target) );
 	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
 }
 
@@ -66,8 +66,9 @@ int mk_int_mul( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *ri
 static 
 int mk_int_div( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result )
 {
-	int newValue = 
-		mk_vm_frame_item_to_int32( target ) / mk_vm_frame_item_to_int32( right );
+	int l = mk_vm_frame_item_to_int32( target );
+	int r= mk_vm_frame_item_to_int32( right );
+	int newValue = ( r != 0 ) ? l / r : 0;
 	*result = mk_vm_create_int32_frame_item( vm, newValue );
 	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
 }
@@ -75,8 +76,9 @@ int mk_int_div( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *ri
 static 
 int mk_int_mod( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result )
 {
-	int newValue = 
-		mk_vm_frame_item_to_int32( target ) % mk_vm_frame_item_to_int32( right );
+	int l = mk_vm_frame_item_to_int32( target );
+	int r = mk_vm_frame_item_to_int32( right );
+	int newValue = ( r != 0 ) ? l % r : 0;
 	*result = mk_vm_create_int32_frame_item( vm, newValue );
 	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
 }
@@ -129,125 +131,168 @@ MK_CLASS *mk_create_int_class( MK_VM_STRUCT *vm )
 	result->nameThis = mk_get_symbol_name_ptr( vm, className );
 	result->nameSuper = mk_get_symbol_name_ptr( vm, classSuper );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"to_s",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE, 
-			mk_int_to_s ),
+			(INT_PTR)mk_int_to_s ),
 		result );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm,
 			"to_i",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE, 
-			mk_int_to_i ),
+			(INT_PTR)mk_int_to_i ),
 		result );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm,
 			"to_f",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE, 
-			mk_int_to_f ),
+			(INT_PTR)mk_int_to_f ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
-			"+", 
+			"+",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_PLUS, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_PLUS, 
-			mk_int_plus ),
+			(INT_PTR)mk_int_plus ),
 		result);
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
 			"-", 
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_MINUS, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_MINUS, 
-			mk_int_minus ),
+			(INT_PTR)mk_int_minus ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
 			"*", 
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_MUL, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_MUL, 
-			mk_int_mul ),
+			(INT_PTR)mk_int_mul ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
 			"/", 
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_DIV, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_DIV, 
-			mk_int_div ),
+			(INT_PTR)mk_int_div ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function( 
+		mk_create_method( 
 			vm, 
 			"%", 
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_MOD, 
 			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_MOD, 
-			mk_int_mod ),
+			(INT_PTR)mk_int_mod ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"<",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_SMALL, 
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_SMALL, 
-			mk_int_small ),
+			(INT_PTR)mk_int_small ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			">",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_BIG, 
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_BIG, 
-			mk_int_large ),
+			(INT_PTR)mk_int_large ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			"<=",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_SE, 
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_SE, 
-			mk_int_small_same ),
+			(INT_PTR)mk_int_small_same ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable( 
 		vm,
-		mk_create_native_function(
+		mk_create_method(
 			vm, 
 			">=",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_BE, 
 			1,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_BE, 
-			mk_int_large_same ),
+			(INT_PTR)mk_int_large_same ),
 		result );
 
 	return result;
-}
\ No newline at end of file
+}
