@@ -7,6 +7,33 @@
 static const MK_CHAR className[] = CLASS_INTERNAL_OBJECT;
 static const MK_CHAR classSuper[] = "";
 
+static
+int mk_object_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result )
+{
+	MK_CLASS *targetClass = 
+		mk_vm_get_class(vm, target );
+	if( targetClass == NULL )
+	{
+		if( MK_TYPE_ATTRIBUTE( (*result)->flags) == MK_VM_FRAME_ITEM_TYPE_CLASS )
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 2, 
+										(*result)->classTypeValue.typeName, NULL );
+		else
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 1, 
+										NULL );
+	}
+
+	if( mk_find_item_hashtable( vm->global, targetClass->nameThis, (void*)result ) == 0 )
+	{
+		return mk_raise_internal_error( vm, "", 0, 
+									MK_ERROR_TYPE_VM_ERROR | 1, 
+									NULL );
+	}
+
+	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
+}
+
 static
 int mk_object_to_s( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result )
 {
@@ -20,9 +47,20 @@ int mk_object_to_s( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM
 
 	pClass = mk_vm_get_class( vm, target );
 	if( pClass == NULL )
-		className = "Unknown";
+	{
+		if( MK_TYPE_ATTRIBUTE( (*result)->flags) == MK_VM_FRAME_ITEM_TYPE_CLASS )
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 2, 
+										target->classTypeValue.typeName, NULL );
+		else
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 1, 
+										NULL );
+	}
 	else
+	{
 		className = pClass->nameThis;
+	}
 
 	// #<className:0xaddress>"
 	length = sizeof(MK_CHAR) * ( 2 + strlen( className ) + 1 + 2 + sizeof( void* ) * 2 + 1 + 1 );
@@ -31,27 +69,54 @@ int mk_object_to_s( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM
 	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
 }
 
-static 
 int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result )
 {
+	int retCode = MK_VM_EXECUTE_EXPR_RETURN_RETURN;
+	MK_VM_FRAME *pInitializeFrame = NULL;
 	MK_HASHTABLE *classes = 
 		vm->code->classes;
 	MK_CLASS *thisClass = NULL, *currentClass = NULL;
 	MK_VM_FRAME_ITEM *currentFrameItem = NULL, *previousFrameItem = NULL;
 	*result = NULL;
+	
+	if( ( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE ) ||
+		MK_TYPE_ATTRIBUTE(target->flags) != MK_VM_FRAME_ITEM_TYPE_CLASS ||
+		!( target->flags & MK_VM_FRAME_ITEM_TYPE_STATIC_INSTANCE ) )
+	{
+		return mk_raise_internal_error( vm, "", 0, 
+									MK_ERROR_TYPE_VM_ERROR | 12, 
+									"new", NULL );
+	}
 
-	if( MK_TYPE_ATTRIBUTE(target->flags) != MK_VM_FRAME_ITEM_TYPE_CLASS )
-		return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
-	mk_find_item_hashtable( classes, target->classTypeValue.typeName, (void**)&thisClass );
+	thisClass = mk_vm_get_class( vm, target );
 	if( thisClass == NULL )
-		return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
+	{
+		if( MK_TYPE_ATTRIBUTE( (*result)->flags) == MK_VM_FRAME_ITEM_TYPE_CLASS )
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 2, 
+										target->classTypeValue.typeName, NULL );
+		else
+			return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 1, 
+										NULL );
+	}
 
 	currentClass = 
 		thisClass;
 
+	pInitializeFrame = 
+		mk_create_vm_frame_object( &vm->pFrameTable );
+	pInitializeFrame->localVariables = NULL;
+	pInitializeFrame->pMethod = vm->pCurrentFrame->pMethod;
+	pInitializeFrame->pThis = *result;
+	pInitializeFrame->pOwnerFrame = NULL;
+	pInitializeFrame->previous = vm->pCurrentFrame;
+	vm->pCurrentFrame = pInitializeFrame;
+
 	// setup variables
 	while(currentClass)
 	{
+
 		const MK_CHAR *key = NULL;
 		MK_VARIABLE* value = NULL;
 		MK_VM_FRAME_ITEM *insertValue = NULL;
@@ -67,6 +132,8 @@ int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM
 		{
 			MK_CHAR *nameSuper = 
 				mk_get_symbol_name_ptr( vm, "super" );
+			currentFrameItem->flags 
+				|= MK_VM_FRAME_ITEM_TYPE_FINAL;
 			mk_insert_item_hashtable( 
 				previousFrameItem->classTypeValue.variables, 
 				nameSuper, 
@@ -84,20 +151,49 @@ int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM
 			while( iterator != NULL )
 			{
 				iterator = mk_enum_item_hashtable_next( currentClass->variables, iterator, &key, (void**)&value );
-				if( MK_TYPE_ATTRIBUTE( value->flags ) == MK_TYPE_ATTRIBUTE_VARIABLE_MEMBER )
+				insertValue = 
+					mk_create_vm_frame_item_object( &vm->pFrameItemTable );
+				if( value->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD )
 				{
-					if( value->defaultValue != NULL )	// todo: set current frame to target class.
+					insertValue->flags |= 
+						MK_VM_FRAME_ITEM_TYPE_FINAL | MK_VM_FRAME_ITEM_TYPE_NODE_VALUE;
+					insertValue->code.definedFrame = NULL;
+					insertValue->code.pOwner = currentFrameItem;
+					insertValue->code.node = (unsigned int*)value;
+					mk_insert_item_hashtable( 
+						currentFrameItem->classTypeValue.variables, 
+						key, 
+						insertValue );
+				}
+				else if( !( value->flags & MK_TYPE_ATTRIBUTE_VARIABLE_STATIC ) )
+				{
+					if( value->defaultValue != NULL )
 					{
 						int retCodeDefaultValue = 
 							mk_execute_expr( vm, value->defaultValue, 0 );
 						if( retCodeDefaultValue == MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
-							insertValue = mk_pop_vector( vm->localStack );
+						{
+							retCodeDefaultValue = 
+								mk_object_equal( vm, 
+									insertValue, 
+									(MK_VM_FRAME_ITEM*)mk_vm_pop_stack( &vm->localStack ), 
+									&insertValue );
+							if( retCodeDefaultValue != MK_VM_EXECUTE_EXPR_RETURN_RETURN )
+							{
+								retCode = retCodeDefaultValue;
+								break;
+							}
+						}
 						else
-							return retCodeDefaultValue;
+						{
+							retCode = retCodeDefaultValue;
+							break;
+						}
 					}
 					else
 					{
-						insertValue = mk_create_vm_frame_item_object( &vm->pFrameItemTable );
+						insertValue = 
+							mk_create_vm_frame_item_object( &vm->pFrameItemTable );
 					}
 					mk_insert_item_hashtable( 
 						currentFrameItem->classTypeValue.variables, 
@@ -110,7 +206,8 @@ int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM
 		previousFrameItem = currentFrameItem;
 		mk_find_item_hashtable( classes, currentClass->nameSuper, (void**)&currentClass );
 	}
-	return MK_VM_EXECUTE_EXPR_RETURN_RETURN;
+	vm->pCurrentFrame = vm->pCurrentFrame->previous;
+	return retCode;
 }
 
 static 
@@ -169,33 +266,15 @@ int mk_object_equal_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRA
 		switch( MK_TYPE_ATTRIBUTE(target->flags) )
 		{
 		case MK_VM_FRAME_ITEM_TYPE_CLASS:
-			
-			if( strcmp( target->classTypeValue.typeName, right->classTypeValue.typeName ) == 0 )
-				break;
-			if( mk_size_item_hashtable( target->classTypeValue.variables ) !=
-				mk_size_item_hashtable( target->classTypeValue.variables ) )
-				break;
-			position = mk_enum_item_hashtable_begin( target->classTypeValue.variables );
-			while( position )
-			{
-				MK_VM_FRAME_ITEM *childValue = NULL;
-				if( mk_is_key_hashtable( target->classTypeValue.variables, key ) != 
-					mk_is_key_hashtable( target->classTypeValue.variables, key ) )
-					break;
-
-				position = 
-					mk_enum_item_hashtable_next( target->classTypeValue.variables, position, &key, (void**)&value );
-				mk_find_item_hashtable( right->classTypeValue.variables, key, (void**)&rightValue );
-				retCode = 
-					mk_object_equal_equal( vm, value, rightValue, &childValue );
-				if( retCode != MK_VM_EXECUTE_EXPR_RETURN_RETURN )
-					break;
-				if( childValue == NULL || childValue->int32TypeValue == 0 )
-					break;
-				if( position == NULL )
-					isTrue = 1;
-			}
-			mk_enum_item_hashtable_end( target->classTypeValue.variables, position );
+			// get top level object.
+			while( target->classTypeValue.child != NULL )
+				target = target->classTypeValue.child;
+			while( right->classTypeValue.child != NULL )
+				right = right->classTypeValue.child;
+			if( memcmp( &target->classTypeValue, 
+						&right->classTypeValue, 
+						sizeof( target->classTypeValue ) ) == 0 )
+				isTrue = 1;
 			break;
 
 		case MK_VM_FRAME_ITEM_TYPE_NIL:
@@ -225,12 +304,25 @@ int mk_object_equal_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRA
 	return retCode;
 }
 
-static int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result )
+int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result )
 {
 	int retCode = MK_VM_EXECUTE_EXPR_RETURN_RETURN;
 	if( target->flags & MK_VM_FRAME_ITEM_TYPE_FINAL )
-		return mk_raise_internal_error( vm, "", 0, MK_ERROR_TYPE_VM_ERROR | 13, NULL );
+	{
+		return mk_raise_internal_error( vm, "", 0, 
+										MK_ERROR_TYPE_VM_ERROR | 10, 
+										NULL );	// final object
+	}
 
+	//todo: [interim] return Object class
+	if( MK_TYPE_ATTRIBUTE(target->flags) == MK_VM_FRAME_ITEM_TYPE_STRING_VALUE )
+	{
+		if( !mk_is_ptr_in_memory_pool( target->stringTypeValue, vm->memoryPool ) )
+		{
+			free( target->stringTypeValue );
+			target->stringTypeValue = NULL;
+		}
+	}
 	while( ( (INT_PTR)right & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE ) == 0 &&
 		MK_TYPE_ATTRIBUTE( right->flags ) == MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE )
 		right = right->reference;
@@ -268,6 +360,8 @@ static int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FR
 		switch( MK_TYPE_ATTRIBUTE( right->flags ) )
 		{
 		case MK_VM_FRAME_ITEM_TYPE_CLASS:
+			while( right->classTypeValue.child != NULL )
+				right = right->classTypeValue.child;
 			target->classTypeValue.typeName = 
 				mk_get_symbol_name_ptr( vm, right->classTypeValue.typeName );
 			target->classTypeValue.variables = right->classTypeValue.variables;
@@ -289,6 +383,16 @@ static int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FR
 		case MK_VM_FRAME_ITEM_TYPE_STRING_VALUE:
 			mk_copy_string( &target->stringTypeValue, right->stringTypeValue );
 			break;
+
+		case MK_VM_FRAME_ITEM_TYPE_ARRAY_VALUE:
+			target->arrayTypeValue = right->arrayTypeValue;
+			break;
+
+		case MK_VM_FRAME_ITEM_TYPE_NODE_VALUE:
+			target->code.node = right->code.node;
+			target->code.pOwner = right->code.pOwner;
+			target->code.definedFrame = right->code.definedFrame;
+			break;
 		}
 	}
 	*result = target;
@@ -302,7 +406,7 @@ int mk_object_not_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME
 		mk_object_equal_equal( vm, target, right, result );
 	if( retCode == MK_VM_EXECUTE_EXPR_RETURN_RETURN )
 		*result = 
-			mk_vm_create_bool_frame_item( mk_vm_frame_item_is_true( *result ) );
+			mk_vm_create_bool_frame_item( !mk_vm_frame_item_is_true( *result ) );
 	return retCode;
 }
 
@@ -310,59 +414,89 @@ MK_CLASS *mk_create_object_class( MK_VM_STRUCT *vm )
 {
 	MK_CLASS *result =
 		mk_create_object( MK_TYPE_CLASS );
-
+	MK_VARIABLE *insertMethod = NULL;
 	result->nameThis = mk_get_symbol_name_ptr( vm, className );
 	result->nameSuper = mk_get_symbol_name_ptr( vm, classSuper );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
-			vm, 
+		mk_create_method(
+			vm,
+			"class",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
+			0,
+			(INT_PTR)mk_object_class ),
+		result );
+
+	mk_register_variable(
+		vm,
+		mk_create_method(
+			vm,
 			"to_s",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_STATIC |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE, 
-			mk_object_to_s ),
+			(INT_PTR)mk_object_to_s ),
 		result );
 
-	mk_register_function(
+	mk_register_variable(
 		vm,
-		mk_create_native_function(
-			vm, 
+		mk_create_method(
+			vm,
 			"new",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE,
 			0,
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE, 
-			mk_object_new ),
+			(INT_PTR)mk_object_new ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable(
 		vm,
-		mk_create_native_function( 
-			vm, 
-			"=", 
-			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_EQUAL, 
-			mk_object_equal ),
+		mk_create_method(
+			vm,
+			"=",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_EQUAL, 
+			1,
+			(INT_PTR)mk_object_equal ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable(
 		vm,
-		mk_create_native_function( 
-			vm, 
-			"==", 
-			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_SAME, 
-			mk_object_equal_equal ),
+		mk_create_method(
+			vm,
+			"==",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_SAME, 
+			1,
+			(INT_PTR)mk_object_equal_equal ),
 		result );
 
-	mk_register_function( 
+	mk_register_variable(
 		vm,
-		mk_create_native_function( 
-			vm, 
-			"!=", 
-			1, 
-			MK_TYPE_FUNCTION | MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE | MK_TYPE_ATTRIBUTE_FUNCTION_OPERATOR | MK_LEX_TYPE_RESERVED_MARK_NOT_SAME, 
-			mk_object_not_equal ),
+		mk_create_method(
+			vm,
+			"!=",
+			MK_TYPE_VARIABLE | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD | 
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE |
+				MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_OPERATOR |
+				MK_LEX_TYPE_RESERVED_MARK_NOT_SAME, 
+			1,
+			(INT_PTR)mk_object_not_equal ),
 		result );
 
 	return result;
-}
\ No newline at end of file
+}
