@@ -22,19 +22,62 @@ void dump_vector_node( MK_VECTOR *target, unsigned int rank )
 	fprintf( stdout, "MK_VECTOR\n" );
 	if( target == NULL )return;
 
+	print_space( rank );
+	switch( MK_TYPE_ATTRIBUTE_VECTOR_STYLE(target->flags) )
+	{
+	case MK_TYPE_VECTOR_ARRAY_MANAGED_PTR:
+		fprintf( stdout, "(MANAGED_PTR)\n" );
+		break;
+	case MK_TYPE_VECTOR_ARRAY_UNMANAGED_PTR:
+		fprintf( stdout, "(UNMANAGED_PTR)\n" );
+		break;
+	case MK_TYPE_VECTOR_ARRAY_INT8:
+		fprintf( stdout, "(INT8)\n" );
+		break;
+	case MK_TYPE_VECTOR_ARRAY_INT16:
+		fprintf( stdout, "(INT16)\n" );
+		break;
+	case MK_TYPE_VECTOR_ARRAY_INT32:
+		fprintf( stdout, "(INT32)\n" );
+		break;
+	case MK_TYPE_VECTOR_ARRAY_INT64:
+		fprintf( stdout, "(INT64)\n" );
+		break;
+	}
+
 	size = 	target->used;
 	for( index = 0; index < size; index ++ )
 	{
 		print_space( rank );
-		fprintf( stdout, "ITEM(%d)\n", index );
-		if( target->items[index] != NULL )
+		switch( MK_TYPE_ATTRIBUTE_VECTOR_STYLE(target->flags) )
 		{
-			dump_node( target->items[index], rank + 1 );
-		}
-		else
-		{
-			print_space( rank + 1 );
-			fprintf( stdout, "( NULL )\n", index );
+		case MK_TYPE_VECTOR_ARRAY_MANAGED_PTR:
+		case MK_TYPE_VECTOR_ARRAY_UNMANAGED_PTR:
+			{
+				fprintf( stdout, "ITEM(%d)\n", index );
+				if( target->items[index] != (INT_PTR)NULL )
+				{
+					dump_node( (void*)target->items[index], rank + 1 );
+				}
+				else
+				{
+					print_space( rank + 1 );
+					fprintf( stdout, "( NULL )\n", index );
+				}
+			}
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT8:
+			fprintf( stdout, "%02x ", target->items8[index] );
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT16:
+			fprintf( stdout, "%04x ", target->items16[index] );
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT32:
+			fprintf( stdout, "%08x ", target->items32[index] );
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT64:
+			fprintf( stdout, "%016x ", target->items64[index] );
+			break;
 		}
 	}
 
@@ -96,6 +139,18 @@ void dump_expr( MK_NODE_EXPR *target, unsigned int rank )
 				dump_node( target->u2.args, rank + 1 );
 			break;
 		
+		case MK_TYPE_NODE_EXPR_NODE_WITH_PARAM:
+			print_space( rank );
+			fprintf( stdout,"NODE_WITH_PARAM\n" );
+			dump_node( target->u1.node, rank + 1 );
+			break;
+		
+		case MK_TYPE_NODE_EXPR_ARRAY_DEFINITION:
+			print_space( rank );
+			fprintf( stdout, "DEF_ARRAY\n" );
+			dump_vector_node( target->u1.arrayDefinition, rank + 1 );
+			break;
+
 		case MK_TYPE_NODE_EXPR_MULTIPLESYMBOL:
 			print_space( rank );
 			fprintf( stdout, "SYMBOL(M):\n" );
@@ -233,48 +288,44 @@ void dump_node( void *target, unsigned int rank )
 				classTarget->nameThis, 
 				( classTarget->nameSuper == NULL ) ? "" : classTarget->nameSuper );
 			print_space( rank );
-			fprintf( stdout, "VARIABLES\n" );
+			fprintf( stdout, "MEMBERS\n" );
 			dump_node( classTarget->variables, rank + 1 );
 			print_space( rank );
-			fprintf( stdout, "FUNCTIONS\n" );
-			dump_node( classTarget->functions, rank + 1 );
 		}
 		break;
 	
-	case MK_TYPE_FUNCTION:
-		{
-			MK_FUNCTION *functionTarget = 
-				( MK_FUNCTION *)target;
-			print_space( rank );
-			fprintf( stdout, "FUNCTION:%s\n", functionTarget->name );
-			print_space( rank );
-			if( functionTarget->flags & MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE )
-			{
-				print_space( rank );
-				fprintf( stdout, "ARG_SIZE:%d\n", functionTarget->sizeArgs );
-				print_space( rank );
-				fprintf( stdout, "NATIVE:0x%08p\n", functionTarget->impl );
-			}
-			else
-			{
-				print_space( rank );
-				fprintf( stdout, "ARG\n" );
-				dump_node( functionTarget->args, rank + 1 );
-				print_space( rank );
-				fprintf( stdout, "IMPL\n" );
-				dump_node( functionTarget->impl, rank + 1 );
-			}
-		}
-		break;
-
 	case MK_TYPE_VARIABLE:
 		{
 			MK_VARIABLE *variableTarget = 
 				( MK_VARIABLE * )target;
-			print_space( rank );
-			fprintf( stdout, "VARIABLE:%s\n", 
-				variableTarget->name );
-			dump_node( variableTarget->defaultValue, rank + 1 );
+			if( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD )
+			{
+				print_space( rank );
+				fprintf( stdout, "FUNCTION:%s\n", variableTarget->name );
+				if( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE )
+				{
+					print_space( rank );
+					fprintf( stdout, "ARG_SIZE:%d\n", variableTarget->sizeArgs );
+					print_space( rank );
+					fprintf( stdout, "NATIVE:0x%08p\n", variableTarget->entryPoint );
+				}
+				else
+				{
+					print_space( rank );
+					fprintf( stdout, "ARG\n" );
+					dump_node( variableTarget->args, rank + 1 );
+					print_space( rank );
+					fprintf( stdout, "IMPL\n" );
+					dump_node( variableTarget->impl, rank + 1 );
+				}
+			}
+			else
+			{
+				print_space( rank );
+				fprintf( stdout, "VARIABLE:%s\n", 
+					variableTarget->name );
+				dump_node( variableTarget->defaultValue, rank + 1 );
+			}
 		}
 		break;
 	
@@ -354,7 +405,7 @@ void dump_node( void *target, unsigned int rank )
 				dump_node( pFrame->localVariables, rank + 1 );
 				print_space( rank + 1 );
 				fprintf( stdout, "CurrentFunction:\n" );
-				dump_node( pFrame->pFunction, rank + 1 );
+				dump_node( pFrame->pMethod, rank + 1 );
 				pFrame = pFrame->previous;
 			}
 		}
@@ -405,31 +456,23 @@ void dump_node( void *target, unsigned int rank )
 				case MK_VM_FRAME_ITEM_TYPE_STRING_VALUE:
 					fprintf( stdout, "STRING_TYPE(\"%s\")\n", pFrameItem->stringTypeValue );
 					break;
+				case MK_VM_FRAME_ITEM_TYPE_ARRAY_VALUE:
+					fprintf( stdout, "ARRAY_TYPE:\n" );
+					dump_node( pFrameItem->arrayTypeValue, rank + 1 );
+					break;
+				default:
+					fprintf( stdout, "\n" );
+					break;
 				}
 			}
 		}
 		break;
 	
-	case MK_TYPE_NODE_WITH_PARAM:
-		{
-			MK_NODE_WITH_PARAM *param = 
-				( MK_NODE_WITH_PARAM * )target;
-			print_space( rank );
-			fprintf( stdout, "NODE_WITH_PARAM:\n" );
-			print_space( rank + 1 );
-			fprintf( stdout, "ARGUMENTS:\n" );
-			dump_vector_node( param->param, rank + 1 );
-			print_space( rank + 1 );
-			fprintf( stdout, "NODE:\n" );
-			dump_node( param->node, rank + 1 );
-		}
-		break;
-
 	case MK_TYPE_TRY_BLOCK:
 		{
 			MK_TRY_BLOCK *tryBlock = 
 				( MK_TRY_BLOCK * )target;
-			int size = mk_size_vector( tryBlock->blockCatch );
+			int size = tryBlock->blockCatch != NULL ? mk_size_vector( tryBlock->blockCatch ) : 0;
 			int index = 0;
 			print_space( rank );
 			fprintf( stdout, "TRY:\n" );
@@ -437,19 +480,8 @@ void dump_node( void *target, unsigned int rank )
 			for( index = 0; index < size; index ++ )
 			{
 				MK_CATCH_BLOCK *catchBlock = 
-					mk_get_at_vector( tryBlock->blockCatch, index );
-				if( catchBlock->paramCatch == NULL )
-				{
-					print_space( rank );
-					fprintf( stdout, "ELSE:\n" );
-				}
-				else
-				{
-					print_space( rank );
-					fprintf( stdout, "CATCH:\n" );
-					dump_node( catchBlock->paramCatch, rank + 1 );
-				}
-				dump_node( catchBlock->blockCatch, rank + 1 );
+					(MK_CATCH_BLOCK *)mk_get_at_vector( tryBlock->blockCatch, index );
+				dump_node( catchBlock, rank + 1 );
 			}
 			print_space( rank );
 			fprintf( stdout, "FINALLY:\n" );
@@ -457,6 +489,24 @@ void dump_node( void *target, unsigned int rank )
 			
 		}
 		break;
+	case MK_TYPE_CATCH_BLOCK:
+		{
+			MK_CATCH_BLOCK *catchBlock = 
+				(MK_CATCH_BLOCK*)target;
+			if( catchBlock->paramCatch == NULL )
+			{
+				print_space( rank );
+				fprintf( stdout, "ELSE:\n" );
+			}
+			else
+			{
+				print_space( rank );
+				fprintf( stdout, "CATCH:\n" );
+				dump_node( catchBlock->paramCatch, rank + 1 );
+			}
+			dump_node( catchBlock->blockCatch, rank + 1 );
+		}
+		break;
 
 	case MK_OBJECT_TYPE_VM_STRUCT:
 		{
@@ -469,7 +519,7 @@ void dump_node( void *target, unsigned int rank )
 			dump_node( pStruct->pCurrentFrame, rank + 1 );
 			print_space( rank + 1 );
 			fprintf( stdout, "LocalStack:\n" );
-			dump_node( pStruct->localStack, rank + 1 );
+//			dump_node( pStruct->localStack, rank + 1 );
 		}
 		break;
 
