@@ -66,7 +66,7 @@ int main( int argc, char **argv )
 
 	if( argc < 2 )
 	{
-		fprintf( stdout, "%s  Biuld - %s %s\n", argv[0], __DATE__, __TIME__ );
+		fprintf( stdout, "%s  Build - %s %s\n", argv[0], __DATE__, __TIME__ );
 		return 0;
 	}
 
@@ -84,13 +84,12 @@ int main( int argc, char **argv )
 		{
 			MK_VM_FRAME_ITEM *pErrorDescription = NULL;
 			mk_trace( MK_TRACE_TYPE_ERROR, "unhandle error." );
-			mk_find_item_hashtable( vm->exceptionObject->classTypeValue.variables, 
-								   mk_get_symbol_name_ptr( vm, "description" ), 
-								   (void**)&pErrorDescription );
+			pErrorDescription  = 
+				mk_vm_find_variable( vm, vm->exceptionObject, mk_get_symbol_name_ptr( vm, "description" ) );
 			if( pErrorDescription != NULL )
 				mk_trace( MK_TRACE_TYPE_ERROR, pErrorDescription->stringTypeValue );
 		}
-#ifdef _DEBUG
+#ifdef _DEBUG_DUMP
 		show_profile_data( vm );
 #endif
 	}
