@@ -1,7 +1,11 @@
 #include "mk.h"
 #include <stdlib.h>
 #include <string.h>
+#ifdef __APPLE__
+#include <sys/malloc.h>
+#else 
 #include <malloc.h>
+#endif
 
 #define free_if_not_memory_pool( p )				\
 {													\
@@ -180,7 +184,7 @@ MK_VM_FRAME *mk_create_vm_frame_object( MK_MANAGED_VM_FRAME_TABLE **frameTable )
 	MK_GC_SET_MANAGED_BIT( &ptr->flags );
 	MK_GC_ALLOCATE_OBJECT( &ptr->flags );
 	MK_GC_SET_CN_BIT( &ptr->flags );
-	// initialize after this function ptr->previous, ptr->pFunction, ptr->pThis
+	// initialize after this function ptr->previous, ptr->pMethod, ptr->pThis
 	current->freeSpace --;
 	current->nextIndex = ptr - current->table;
 
@@ -209,9 +213,6 @@ void *mk_create_object( unsigned int nsType )
 	case MK_TYPE_CLASS:
 		size = sizeof( MK_CLASS );
 		break;
-	case MK_TYPE_FUNCTION:
-		size = sizeof( MK_FUNCTION );
-		break;
 	case MK_TYPE_VARIABLE:
 		size = sizeof( MK_VARIABLE );
 		break;
@@ -230,8 +231,8 @@ void *mk_create_object( unsigned int nsType )
 	case MK_TYPE_TRY_BLOCK:
 		size = sizeof( MK_TRY_BLOCK );
 		break;
-	case MK_TYPE_NODE_WITH_PARAM:
-		size = sizeof( MK_NODE_WITH_PARAM );
+	case MK_TYPE_CATCH_BLOCK:
+		size = sizeof( MK_CATCH_BLOCK );
 		break;
 	default:
 		size = 0;
@@ -370,9 +371,13 @@ void mk_destroy_vector_node( MK_MEMORY_POOL *top, MK_VECTOR *target )
 		unsigned int size = 0;
 		unsigned int index = 0;
 		size = target->used;
-		for( index = 0; index < size; index ++ )
-			mk_destroy_node( top, target->items[index] );
-		free_if_not_memory_pool( target->items );
+		if( MK_TYPE_ATTRIBUTE_VECTOR_STYLE(target->flags) == MK_TYPE_VECTOR_ARRAY_MANAGED_PTR )
+		{
+			for( index = 0; index < size; index ++ )
+				mk_destroy_node( top, (void*)target->items[index] );
+		}
+		if( ( target->flags & MK_TYPE_VECTOR_POINT_MEMORY_DIRECT ) == 0 )
+			free_if_not_memory_pool( target->items );
 		free_if_not_memory_pool( target );
 	}
 }
@@ -433,12 +438,18 @@ void mk_destroy_expr( MK_MEMORY_POOL *top, MK_NODE_EXPR *target )
 		switch( attribute )
 		{
 		case MK_TYPE_NODE_EXPR_FUNCTION_CALL:
+		case MK_TYPE_NODE_EXPR_FUNCTION_CALL_INSTANCE:
+		case MK_TYPE_NODE_EXPR_FUNCTION_CALL_STATIC:
 		case MK_TYPE_NODE_EXPR_NEW:
 		case MK_TYPE_NODE_EXPR_SUPER_CALL:
 			free_if_not_memory_pool( target->u1.symbolName );
 			mk_destroy_vector_node( top, target->u2.args );
 			break;
 
+		case MK_TYPE_NODE_EXPR_NODE_WITH_PARAM:
+			mk_destroy_node( top, target->u1.node );
+			break;
+
 		case MK_TYPE_NODE_EXPR_SYMBOL:
 		case MK_TYPE_NODE_EXPR_ATSYMBOL:
 		case MK_TYPE_NODE_EXPR_DBLATSYMBOL:
@@ -493,6 +504,10 @@ void mk_destroy_expr( MK_MEMORY_POOL *top, MK_NODE_EXPR *target )
 		case MK_TYPE_NODE_EXPR_FLOAT:
 		case MK_TYPE_NODE_EXPR_CHAR:
 			break;
+		
+		case MK_TYPE_NODE_EXPR_ARRAY_DEFINITION:
+			mk_destroy_vector_node( top, target->u1.arrayDefinition );
+			break;
 		}
 		break;
 	}
@@ -521,11 +536,12 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 			MK_MANAGED_VM_FRAME_TABLE *pFrameTable = vm->pFrameTable;
 			MK_MANAGED_VM_FRAME_ITEM_TABLE *pTable = vm->pFrameItemTable;
 			MK_MANAGED_HASH_TABLE *pHash = vm->pHashTable;
+			MK_MANAGED_VECTOR_TABLE *pVector = vm->pVectorTable;
 			MK_MEMORY_POOL *memoryPool = vm->memoryPool;
 			mk_destroy_node( top, vm->code );
 			free_if_not_memory_pool( vm->cache );
 			mk_destroy_hashtable_node( top, vm->hashSymbolName, 0 );
-			mk_destroy_vector_node( top, vm->localStack );
+			free_if_not_memory_pool( vm->localStack.start );
 			while( pTable )
 			{
 				int index = 0;
@@ -553,16 +569,34 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 				{
 					MK_HASHTABLE *pTarget = 
 						&pHash->hashTables[index];
-					if( MK_GC_IS_ALLOCATED_OBJECT( pTarget->flags ) )
+					if( pTarget->elems != NULL )
 					{
-						mk_destroy_map_item( top, pTarget->elems, pTarget->used, 0 );
+						if( MK_TYPE_ATTRIBUTE_HASHKEY_STYLE( pTarget->flags ) == 
+							MK_TYPE_ATTRIBUTE_HASHKEY_STRING )
+							mk_destroy_map_item( top, pTarget->elems, pTarget->used, 0 );
 						free_if_not_memory_pool( pTarget->elems );
+						pTarget->elems = NULL;
 					}
 				}
 				pNext = pHash->previous;
 				free_if_not_memory_pool( pHash );
 				pHash = pNext;
 			}
+			while( pVector )
+			{
+				int index = 0;
+				MK_MANAGED_VECTOR_TABLE *pNext = NULL;
+				for( index = 0; index < MK_SIZEOF_MANAGED_TABLE; index ++ )
+				{
+					MK_VECTOR *pTarget = 
+						&pVector->vectors[index];
+					if( MK_GC_IS_ALLOCATED_OBJECT( pTarget->flags ) )
+						free_if_not_memory_pool( pTarget->items );
+				}
+				pNext = pVector->previous;
+				free_if_not_memory_pool( pVector );
+				pVector = pNext;
+			}
 			while( memoryPool )
 			{
 				MK_MEMORY_POOL *previous = memoryPool->previous;
@@ -594,32 +628,36 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 			MK_CLASS *classTarget = 
 				( MK_CLASS * )target;
 			mk_destroy_hashtable_node( top, classTarget->variables, 1 );
-			mk_destroy_hashtable_node( top, classTarget->functions, 1 );
+			if( classTarget->operatorMethod != NULL )
+			{
+				int index = 0;
+				for( index = 0; index < MK_SIZEOF_ENABELD_OPELATORS; index ++ )
+					mk_destroy_node( top, (void*)classTarget->operatorMethod[index] );
+				free_if_not_memory_pool( classTarget->operatorMethod );
+			}
 			free_if_not_memory_pool( classTarget->nameThis );
 			free_if_not_memory_pool( classTarget->nameSuper );
-			mk_destroy_vector_node( top, classTarget->operatorFunctions );
 		}
 		break;
 	
-	case MK_TYPE_FUNCTION:
-		{
-			MK_FUNCTION *functionTarget = 
-				( MK_FUNCTION *)target;
-			free_if_not_memory_pool( functionTarget->name );
-			if( !( functionTarget->flags & MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE ) )
-			{
-				mk_destroy_vector_node( top, functionTarget->args );
-				mk_destroy_node( top, functionTarget->impl );
-			}
-		}
-		break;
-
 	case MK_TYPE_VARIABLE:
 		{
 			MK_VARIABLE *variableTarget = 
 				( MK_VARIABLE * )target;
-			mk_destroy_node( top, variableTarget->defaultValue );
-			free_if_not_memory_pool( variableTarget->name );
+			if( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD )
+			{	// method
+				if( !( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE ) &&
+					!( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_VIRTUAL ) )
+				{
+					mk_destroy_vector_node( top, variableTarget->args );
+					mk_destroy_node( top, variableTarget->impl );
+				}
+			}
+			else
+			{	// value
+				mk_destroy_node( top, variableTarget->defaultValue );
+				free_if_not_memory_pool( variableTarget->name );
+			}
 		}
 		break;
 	
@@ -661,36 +699,31 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 		mk_destroy_expr( top, target );
 		break;
 
-	case MK_TYPE_NODE_WITH_PARAM:
-		{
-			MK_NODE_WITH_PARAM *node = 
-				( MK_NODE_WITH_PARAM* )target;
-			mk_destroy_vector_node( top, node->param );
-			mk_destroy_node( top, node->node );
-		}
-		break;
 	case MK_TYPE_TRY_BLOCK:
 		{
-			MK_CATCH_BLOCK *catchBlock = NULL;
 			MK_TRY_BLOCK *tryBlock = 
 				( MK_TRY_BLOCK *) target;
-			unsigned int size = mk_size_vector( tryBlock->blockCatch );
-			unsigned int index = 0;
 			mk_destroy_node( top, tryBlock->blockTry );
-			for( index = 0; index < size; index ++ )
-			{
-				catchBlock = 
-					( MK_CATCH_BLOCK * )mk_get_at_vector( tryBlock->blockCatch, index );
-				mk_destroy_node( top, catchBlock->paramCatch );
-				mk_destroy_node( top, catchBlock->blockCatch );
-				free_if_not_memory_pool( catchBlock );
-			}
-			mk_trim_size_vector( tryBlock->blockCatch, 0 );
 			mk_destroy_vector_node( top, tryBlock->blockCatch );
 			mk_destroy_node( top, tryBlock->blockNoException );
 			mk_destroy_node( top, tryBlock->blockFinally );
 		}
 		break;
+	
+	case MK_TYPE_CATCH_BLOCK:
+		{
+			MK_CATCH_BLOCK *catchBlock = 
+				(MK_CATCH_BLOCK*) target;
+			if(  catchBlock->blockCatch != NULL )
+			{
+				MK_VARIABLE *variable = 
+					( MK_VARIABLE * )( catchBlock->blockCatch->u1.node );
+				variable->args = NULL;
+			}
+			mk_destroy_node( top, catchBlock->paramCatch );
+			mk_destroy_node( top, catchBlock->blockCatch );
+		}
+		break;
 
 	case MK_TYPE_HASHTABLE:	// call	mk_destroy_hashtable_node, mk_destroy_map_item
 	case MK_TYPE_VECTOR:	// call mk_destroy_vector_node
